from __future__ import print_function, absolute_import, division

import unittest
import numpy as np
import scipy.linalg

from numba.testing.ddt import ddt, unpack, data
import time


@ddt
class TestBLAS:
    """Create test cases by deriving from this (as well as unittest.TestCase.
    In the derived class, set the 'blas' attribute to the BLAS module that is
    to be tested."""

    blas = None

    @data(np.float32, np.float64, np.complex64, np.complex128)
    def test_dot(self, dtype):

        # TODO: Test with different strides
        x = np.arange(1024, dtype=dtype)
        y = np.arange(1024, dtype=dtype)
        res = self.blas.dot(x, y)
        ref = np.dot(x, y)
        np.testing.assert_allclose(res, ref, rtol=1e-6)

    @data(np.float32, np.float64, np.complex64, np.complex128)
    def test_gemv(self, dtype):

        sp_gemv = scipy.linalg.get_blas_funcs('gemv', dtype=dtype)

        # TODO: Test with different strides
        alpha = dtype(2.)
        A = np.arange(64*64, dtype=dtype).reshape(64, 64)
        x = np.arange(64, dtype=dtype)
        beta = dtype(2.)
        y = np.arange(64, dtype=dtype)
        res = self.blas.gemv('N', alpha, A, x, beta, y)
        # scipy.blas expects fortran-style arrays...
        A = np.arange(64*64, dtype=dtype).reshape(64, 64, order='F')
        # ...and y may have been overwritten in the previous operation.
        y = np.arange(64, dtype=dtype)
        ref = sp_gemv(alpha, A, x, beta, y)
        np.testing.assert_allclose(res, ref)

    @data(np.float32, np.float64, np.complex64, np.complex128)
    def test_axpy(self, dtype):

        sp_axpy = scipy.linalg.get_blas_funcs('axpy', dtype=dtype)
        # TODO: Test with different strides
        alpha = dtype(2.)
        x = np.arange(64, dtype=dtype)
        y = np.arange(64, dtype=dtype)
        res = self.blas.axpy(alpha, x, y)
        y = np.arange(64, dtype=dtype)
        ref = sp_axpy(x, y, 64, alpha)
        np.testing.assert_allclose(res, ref)

    @data(np.float32, np.float64, np.complex64, np.complex128)
    def test_gemm(self, dtype):

        sp_gemm = scipy.linalg.get_blas_funcs('gemm', dtype=dtype)

        # TODO: Test with different strides
        N = 64
        alpha = dtype(2.)
        A = np.arange(N*N, dtype=dtype).reshape(N, N)
        B = np.arange(N*N, dtype=dtype).reshape(N, N)
        beta = dtype(5.)
        C = np.arange(N*N, dtype=dtype).reshape(N, N)
        res = self.blas.gemm('N', 'N', alpha, A, B, beta, C)
        # scipy.blas expects fortran-style arrays...
        A = np.arange(N*N, dtype=dtype).reshape(N, N, order='F')
        B = np.arange(N*N, dtype=dtype).reshape(N, N, order='F')
        # ...and C may have been overwritten in the previous operation.
        C = np.arange(N*N, dtype=dtype).reshape(N, N, order='F')
        # For some reason ref != C after the next line, only C
        # appears correct
        ref = sp_gemm(alpha, A, B, beta, C, overwrite_c=True)
        np.testing.assert_allclose(C, ref)

if __name__ == '__main__':
    unittest.main()
